
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <GL/glew.h> // http://glew.sourceforge.net/basic.html

#include <GL/glfw.h> // http://www.glfw.org/GLFWReference27.pdf

// http://www.khronos.org/files/opengl-quick-reference-card.pdf
#include "shaders.h"
#include "geometry.h"
#include "textures.h"

#ifndef M_PI
#define M_PI 3.141592654
#endif

float angleX=0.0;
float angleY=0.0;

int running=1;

//-------------- textures --------------

// Texture IDs
GLuint gColorMapID=0;

// ------------- geometry --------------
SModel modelHead;

// ------------- shaders --------------
GLuint shVertPosID=0;
GLuint shTexCoordID=0;

GLuint shAngleID=0;
GLuint shLightPosID=0;

float gAngleVal=0.0;

GLhandleARB gGLSLprog=0;

void onInit()
{
// load texture
	gColorMapID=loadTGA("./textures/Walker_Head_DF.tga");
// load shaders
    gGLSLprog=createProgram("shaders/minimal.vert","shaders/minimal.frag");

	shVertPosID=glGetAttribLocation(gGLSLprog,"inVertPos");
	shTexCoordID=glGetAttribLocation(gGLSLprog,"inTexCoord");

	loadGeometry(&modelHead);

	glGenBuffers(1, &modelHead.vPosVB); // positions
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.vPosVB);
    glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*3*sizeof(float), modelHead.vPosArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.vPosArr;

	glGenBuffers(1, &modelHead.texCoordVB); // texcoords
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.texCoordVB);
	glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*2*sizeof(float), modelHead.texCoordArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.texCoordArr;

	glGenBuffers(1, &modelHead.triangleIB); // Index Buffer
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,modelHead.triangleIB);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, modelHead.indexCnt*sizeof(unsigned int), modelHead.indexArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.indexArr;
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
}

/* handle rendering */
void onRender()
{
	glDisable(GL_CULL_FACE);
    glEnable(GL_DEPTH_TEST);

	glClearColor(0.4,0.1,0.1,1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gColorMapID);
    glEnable(GL_TEXTURE_2D);

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    glTranslatef( 0.0, 0.0, -2.0 );

    glRotatef(angleY,0,1,0);
	glRotatef(angleX,1,0,0);
// setup shader
    glUseProgramObjectARB(gGLSLprog);
// bind VBO - positions
	glBindBuffer( GL_ARRAY_BUFFER, modelHead.vPosVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shVertPosID,3,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shVertPosID);
// !!! OpenGL < 3.0 requires GL_VERTEX_ARRAY enabled to render geometry 
	glVertexPointer(3,GL_FLOAT,0,0);
	glEnableClientState(GL_VERTEX_ARRAY);

// bind VBO - texcoords
	glBindBufferARB( GL_ARRAY_BUFFER_ARB, modelHead.texCoordVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shTexCoordID,2,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shTexCoordID);

// VBOs done
	glBindBufferARB( GL_ARRAY_BUFFER_ARB, 0);	// unbind ARRAY BUFFER

// bind IB
	glBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, modelHead.triangleIB);	// bind ELEMENT ARRAY BUFFER with glBuffer of indices
	glDrawElements(GL_TRIANGLES,modelHead.indexCnt,GL_UNSIGNED_INT,0);
	glBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, 0);	// unbind ELEMENT ARRAY BUFFER

// clean-up
	glDisableVertexAttribArray(shVertPosID);
	glDisableVertexAttribArray(shTexCoordID);

	glDisableClientState(GL_VERTEX_ARRAY);

	glUseProgramObjectARB(0);
}


/* new window size callback */
void GLFWCALL onReshape( int width, int height )
{
  GLfloat h = (GLfloat) height / (GLfloat) width;
  GLfloat xmax, znear, zfar;

  znear = 0.1f;
  zfar  = 30.0f;
  xmax  = znear*1.0f;

  glViewport( 0, 0, (GLint) width, (GLint) height );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glFrustum( -xmax, xmax, -xmax*h, xmax*h, znear, zfar );
}

/* handle keyboard input */
void GLFWCALL onKey( int k, int action )
{
  if( action != GLFW_PRESS ) return;

  switch (k) {
    case GLFW_KEY_ESC:
        running = 0;
    break;
    default:
        return;
  }
}

/* handle mouse move event */
void GLFWCALL onMouseMove( int x, int y )
{
    int button = GLFW_MOUSE_BUTTON_LEFT;
 /*
 GLFW_MOUSE_BUTTON_LEFT
 GLFW_MOUSE_BUTTON_RIGHT
 GLFW_MOUSE_BUTTON_MIDDLE
 */
    int state = glfwGetMouseButton( button );
    if (state == GLFW_PRESS)
    {
        fprintf(stderr,"%d %d\r",x,y);
    // process
    	angleY=x;
	    angleX=y;
    } 
}

/* program entry */
int main(int argc, char *argv[])
{
    if( !glfwInit() )
    {
        fprintf( stderr, "Failed to initialize GLFW\n" );
        exit( EXIT_FAILURE );
    }
/*
glfwOpenWindow(int width, int height,
    int redbits, int greenbits, int bluebits, int alphabits,
    int depthbits, int stencilbits,
    int mode)
*/
    if( !glfwOpenWindow( 768,768, 0,0,0,0, 16,0, GLFW_WINDOW ) )
    {
        fprintf( stderr, "Failed to open GLFW window\n" );
        glfwTerminate();
        exit( EXIT_FAILURE );
    }

    GLenum err = glewInit();
	if (err != GLEW_OK)
    {
		fprintf(stderr,"Error glew initialization failed (%s).\n",glewGetErrorString(err));
    }
    fprintf(stderr, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));

    glfwSetWindowTitle( "05_shaders" );
    glfwEnable( GLFW_KEY_REPEAT );
    glfwSwapInterval( 1 ); // VSync 1x

    // Set callback functions
    glfwSetWindowSizeCallback( onReshape );

    glfwSetKeyCallback( onKey );
    glfwSetMousePosCallback( onMouseMove );

    onInit();

    // Main loop
    while( running )
    {
         onRender();
        // Swap buffers
        glfwSwapBuffers();
        glfwSleep( 0.01 );    // don't eat whole CPU time
        // Was the window closed?
        if( !glfwGetWindowParam( GLFW_OPENED ) )
        {
            running = 0;
        }
    }

    // Terminate GLFW
    glfwTerminate();

    // Exit program
    exit( EXIT_SUCCESS );
}

